/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mt_init.c
 *
 * Purpose: Initialization functions for the Tcl interface to the MX library.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <tcl.h>
#include <itcl.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_rec.h"
#include "mx_vers.h"
#include "mx_handl.h"
#include "mx_plot.h"
#include "mx_tcl.h"
#include "mx_tclpr.h"

#define USE_ITCL	TRUE

typedef struct {
	char name[80];
	Tcl_ObjCmdProc *fptr;
} MXTCL_COMMAND;

static MXTCL_COMMAND tcl_command[] = {
	{ "mxprivate_init",			Mx_init_cmd },
	{ "mxprivate_get_debug_level",		Mx_get_debug_level_cmd },
	{ "mxprivate_set_debug_level",		Mx_set_debug_level_cmd },
	{ "mxprivate_plotting_is_enabled",	Mx_plotting_is_enabled_cmd },
	{ "mxprivate_set_plot_enable",		Mx_set_plot_enable_cmd },

	{ "mxprivate_install_user_interrupt_handler",
					Mx_install_user_interrupt_handler_cmd },

	{ "mxprivate_install_output_handlers",
					Mx_install_output_handlers_cmd },
	{ "mxprivate_install_dialog_handler",
					Mx_install_dialog_handler_cmd },

	{ "mxprivate_create_record_list",	Mx_create_record_list_cmd },
	{ "mxprivate_read_database",		Mx_read_database_cmd },
	{ "mxprivate_finish_database_initialization",
					Mx_finish_database_initialization_cmd },
	{ "mxprivate_initialize_hardware",	Mx_initialize_hardware_cmd },
	{ "mxprivate_create_record_from_description",
					Mx_create_record_from_description_cmd },
	{ "mxprivate_finish_record_initialization",
					Mx_finish_record_initialization_cmd },
	{ "mxprivate_delete_record",		Mx_delete_record_cmd },
	{ "mxprivate_get_record",		Mx_get_record_cmd },

	{ "mxprivate_get_next_record",		Mx_get_next_record_cmd },
	{ "mxprivate_get_record_name",		Mx_get_record_name_cmd },
	{ "mxprivate_create_description_from_record",
					Mx_create_description_from_record_cmd },
	{ "mxprivate_get_field",		Mx_get_field_cmd },
	{ "mxprivate_set_field",		Mx_set_field_cmd },
	{ "mxprivate_get_object_class_name",	Mx_get_object_class_name_cmd },
	{ "mxprivate_get_version",		Mx_get_version_cmd },

	{ "mxprivate_motor_is_busy",		Mx_motor_is_busy_cmd },
	{ "mxprivate_motor_move_absolute",	Mx_motor_move_absolute_cmd },
	{ "mxprivate_motor_get_position",	Mx_motor_get_position_cmd },
	{ "mxprivate_motor_set_position",	Mx_motor_set_position_cmd },
	{ "mxprivate_motor_soft_abort",		Mx_motor_soft_abort_cmd },
	{ "mxprivate_motor_immediate_abort",	Mx_motor_immediate_abort_cmd },
	{ "mxprivate_motor_positive_limit_hit",
					Mx_motor_positive_limit_hit_cmd },
	{ "mxprivate_motor_negative_limit_hit",
					Mx_motor_negative_limit_hit_cmd },
	{ "mxprivate_motor_find_home_position",
					Mx_motor_find_home_position_cmd },

	{ "mxprivate_scaler_clear",		Mx_scaler_clear_cmd },
	{ "mxprivate_scaler_overflow_set",	Mx_scaler_overflow_set_cmd },
	{ "mxprivate_scaler_read",		Mx_scaler_read_cmd },
	{ "mxprivate_scaler_read_and_clear",	Mx_scaler_read_and_clear_cmd },
	{ "mxprivate_scaler_write",		Mx_scaler_write_cmd },
	{ "mxprivate_scaler_get_dark_current",
					Mx_scaler_get_dark_current_cmd },
	{ "mxprivate_scaler_set_dark_current",
					Mx_scaler_set_dark_current_cmd },

	{ "mxprivate_timer_is_busy",		Mx_timer_is_busy_cmd },
	{ "mxprivate_timer_start",		Mx_timer_start_cmd },
	{ "mxprivate_timer_stop",		Mx_timer_stop_cmd },

	{ "mxprivate_scan_perform_scan",	Mx_scan_perform_scan_cmd },

	{ "mxprivate_send_variable",		Mx_send_variable_cmd },
	{ "mxprivate_receive_variable",         Mx_receive_variable_cmd },
};

static int num_tcl_commands = sizeof(tcl_command) / sizeof(tcl_command[0]);

#if 0
static MXTCL_COMMAND itcl_command[] = {
	{ "Mx::Record::Record-get_field",			Mx_get_field_cmd },
};

static int num_itcl_commands = sizeof(itcl_command) / sizeof(itcl_command[0]);
#endif

static Tcl_Interp *Mxgui_main_interp;

/* The following is the one global variable in libMxTcl. */

MX_HANDLE_TABLE *mxtcl_global_record_handle_table;

/*-------------------------------------------------------*/

MX_EXPORT mx_status_type
mxtcl_process_tcl_events( unsigned long max_events )
{
#if 0
	const char fname[] = "mxtcl_process_tcl_events()";

	unsigned long i;
	int status;

	status = 1;

	for ( i = 0; i < max_events; i++ ) {

		status = Tcl_DoOneEvent( TCL_ALL_EVENTS | TCL_DONT_WAIT );

		if ( status == 0 ) {
			/* No more Tcl events pending, so break out of
			 * the loop.
			 */

			break;
		}
	}
	if ( status != 0 ) {
		/* Must not use mx_error() here because that will result
		 * in mxtcl_process_tcl_events() being called recursively
		 * by the call to Mxgui_error_output_handler() that would
		 * result.  mx_error_quiet will not make a call to
		 * Mxgui_error_output_handler().
		 */
		fprintf(stderr,
			"Some Tcl events have not been processed yet.  "
			"Only the first %lu were processed.", max_events );

		return mx_error_quiet( MXE_LIMIT_WAS_EXCEEDED, fname,
			"Some Tcl events have not been processed yet.  "
			"Only the first %lu were processed.", max_events );
	}
#else
	(void) Tcl_Eval( Mxgui_main_interp, "update" );
#endif

	return MX_SUCCESSFUL_RESULT;
}

/*-------------------------------------------------------*/

MX_EXPORT int
Mxtcl_Init( Tcl_Interp *interp )
{
	return Mx_Init( interp );
}

/*-------------------------------------------------------*/

MX_EXPORT int
Mx_Init( Tcl_Interp *interp )
{
	char version[20];
	int i;

	Mxgui_main_interp = interp;

	/* Register ordinary Tcl commands. */

	for ( i = 0; i < num_tcl_commands; i++ ) {

		Tcl_CreateObjCommand( interp,
				tcl_command[i].name, tcl_command[i].fptr,
				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

	}

#if 0
	/* Register [incr Tcl] commands. */

	for ( i = 0; i < num_itcl_commands; i++ ) {

		Itcl_RegisterObjC( interp,
				itcl_command[i].name, itcl_command[i].fptr,
				(ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

	}
#endif

	/* Declare that we implement part of the Mx package. */

	sprintf(version, "%d.%d",
		mx_get_major_version(), mx_get_minor_version());

	Tcl_PkgProvide( interp, "Mx", version );

	/* Bring the global environment into the context of the libMxTcl
	 * library.  Using getenv() on any variable name seems to do this
	 * as a side effect.
	 */

	(void) getenv( "XYZZY" );

	return TCL_OK;
}

MX_EXPORT int
Mx_init_cmd( ClientData clientData, Tcl_Interp *interp,
		int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_init_cmd()";

	mx_status_type mx_status;

	mx_set_debug_level(0);

	MX_DEBUG( 2,("%s invoked", fname));

#ifdef SIGPIPE
	/* Don't need sigaction() just to ignore a signal. */

	signal( SIGPIPE, SIG_IGN );
#endif

	/* Also trap ctrl-C interrupts. */

	signal( SIGINT, SIG_IGN );

	/* Initialize the subsecond sleep functions (if they need it). */

	mx_msleep(1);

	/* Initialize the drivers for the various MX record types. */

	mx_status = mx_initialize_record_types();

	if ( mx_status.code != MXE_SUCCESS ) {
		return TCL_ERROR;
	}

	/* Create the global handle table for the MX record.  Configure the
	 * table to be incremented in blocks of 100 handles.
	 */

	mx_status = mx_create_handle_table( &mxtcl_global_record_handle_table,
						100, 1 );

	if ( mx_status.code != MXE_SUCCESS ) {
		return TCL_ERROR;
	}

	return TCL_OK;
}

MX_EXPORT int
Mx_get_debug_level_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	int debug_level;

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 0, objv,
		"Usage: get_debug_level" );
		return TCL_ERROR;
	}

	debug_level = mx_get_debug_level();

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, debug_level );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_debug_level_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_set_debug_level_cmd()";

	int debug_level, tcl_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: set_debug_level debug_level" );
		return TCL_ERROR;
	}

	tcl_status = Tcl_GetIntFromObj( interp, objv[1], &debug_level );

	if ( tcl_status != TCL_OK ) {
		if ( objv[1]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as an integer.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as an integer.",
			objv[1]->bytes );
		}
		return TCL_ERROR;
	}

	mx_set_debug_level( debug_level );

	return TCL_OK;
}

MX_EXPORT int
Mx_plotting_is_enabled_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	Tcl_Obj *resultPtr;
	int plot_enable_value;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: get_debug_level handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	plot_enable_value = mx_plotting_is_enabled( record );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, plot_enable_value );

	return TCL_OK;
}

MX_EXPORT int
Mx_set_plot_enable_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_set_plot_enable_cmd()";

	MX_RECORD *record;
	int plot_enable_value, tcl_status;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: set_plot_enable handle plot_enable_value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetIntFromObj( interp, objv[2], &plot_enable_value );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as an integer.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as an integer.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_set_plot_enable( record, plot_enable_value );

	return TCL_OK;
}

