/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mt_scalr.c
 *
 * Purpose: Tcl interface to generic MX scaler functions.
 */

#include <stdio.h>

#include <tcl.h>
#include <itcl.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_rec.h"
#include "mx_scalr.h"
#include "mx_handl.h"
#include "mx_tcl.h"
#include "mx_tclpr.h"

MX_EXPORT int
Mx_scaler_clear_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scaler_clear record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_scaler_clear( record );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_overflow_set_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int overflow_set;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scaler_overflow_set record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_scaler_overflow_set( record, &overflow_set );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, overflow_set );

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_read_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	long value;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scaler_read record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_scaler_read( record, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, value );

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_read_and_clear_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	long value;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scaler_read_and_clear record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_scaler_read_and_clear( record, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetLongObj( resultPtr, value );

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_write_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_scaler_write_cmd()";

	MX_RECORD *record;
	int tcl_status;
	long value;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: scaler_write record_handle value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &value );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as a long.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as a long.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_status = mx_scaler_write( record, value );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_get_dark_current_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double value;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: scaler_get_dark_current record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_scaler_get_dark_current( record, &value );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, value );

	return TCL_OK;
}

MX_EXPORT int
Mx_scaler_set_dark_current_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_scaler_set_dark_current_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double value;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: scaler_set_dark_current record_handle value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &value );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as a double.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as a double.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_status = mx_scaler_set_dark_current( record, value );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

